﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior {

  /// <summary>
  /// An operator that analyzes broken inheritance of wildcards.
  /// </summary>
  [Item("BrokenInheritanceWildcardAnalyzer", "An operator that analyzes broken inheritance of wildcards.")]
  [StorableClass]
  public sealed class BrokenInheritanceWildcardAnalyzer : SingleSuccessorOperator, IAnalyzer {
    private const string ResultsParameterName = "Results";
    private const string PopulationGraphResultName = "PopulationGraph";
    private const string WildcardsResultName = "Wildcards";
    private const string BrokenInteritanceWildcardOccurrenceMatrixParameterName = "BrokenInteritanceWildcardOccurrenceMatrix";

    #region IAnalyzer Members
    public bool EnabledByDefault {
      get { return true; }
    }
    #endregion

    #region Parameter properties
    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters[ResultsParameterName]; }
    }
    #endregion

    #region Properties
    public ResultCollection Results {
      get { return ResultsParameter.ActualValue; }
    }
    #endregion

    [StorableConstructor]
    private BrokenInheritanceWildcardAnalyzer(bool deserializing) : base(deserializing) { }
    private BrokenInheritanceWildcardAnalyzer(BrokenInheritanceWildcardAnalyzer original, Cloner cloner) : base(original, cloner) { }
    public BrokenInheritanceWildcardAnalyzer()
      : base() {
      Parameters.Add(new LookupParameter<ResultCollection>(ResultsParameterName, "The results collection where the analysis values should be stored."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new BrokenInheritanceWildcardAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      var graph = (GenealogyGraph<Permutation>)Results[PopulationGraphResultName].Value;
      var wildcards = (ItemArray<IntArray>)Results[WildcardsResultName].Value;
      var generationZero = graph.Values.Where(x => x.Rank.Contains(0));

      var buildingBlockMatrix = new StringMatrix(wildcards.Count(), generationZero.Count());
      var rowNames = new List<string>();
      var columnNames = new List<string>();
      for (int i = 0; i < wildcards.Count(); i++) {
        var wildcard = wildcards[i];
        rowNames.Add(new Permutation(PermutationTypes.RelativeUndirected, wildcard).ToString());
        for (int j = 0; j < generationZero.Count(); j++) {
          var individual = generationZero.ElementAt(j);
          if (i == 0) columnNames.Add(((Permutation)individual.Data).ToString());
          buildingBlockMatrix[i, j] = AlgorithmBehaviorHelpers.CalculateBrokenInheritanceOccurrences(wildcard, individual);
        }
      }

      buildingBlockMatrix.RowNames = rowNames;
      buildingBlockMatrix.ColumnNames = columnNames;
      Results.Add(new Result(BrokenInteritanceWildcardOccurrenceMatrixParameterName, buildingBlockMatrix));

      return base.Apply();
    }
  }
}
